#include "AMDiS.h"
#include "GNUPlotWriter.h"

using namespace std;
using namespace AMDiS;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return exp(-10.0 * (x * x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dim = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    return -(400.0*r2 - 20.0*dim)*ux;
  };
};

// ===========================================================================
// ===== main program ========================================================
// ===========================================================================

int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: vecmultigrid initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemVec vecmultigrid("vecmultigrid");
  vecmultigrid.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = new AdaptInfo("vecmultigrid->adapt",
				       vecmultigrid.getNumComponents());

  // === create adapt ===
  AdaptStationary *adapt = new AdaptStationary("vecmultigrid->adapt",
					       &vecmultigrid,
					       adaptInfo);
  

  // ===== add boundary conditions =====
  vecmultigrid.addDirichletBC(1, 0, 0, new G);

  // ===== create operators =====
  Operator matrixOperator00(Operator::MATRIX_OPERATOR,
			    vecmultigrid.getFESpace(0),
			    vecmultigrid.getFESpace(0));
  matrixOperator00.addSecondOrderTerm(new Laplace_SOT);
  vecmultigrid.addMatrixOperator(&matrixOperator00, 0, 0);

  Operator rhsOperator0(Operator::VECTOR_OPERATOR,
			vecmultigrid.getFESpace(0));

  int degree = vecmultigrid.getFESpace(0)->getBasisFcts()->getDegree();

  rhsOperator0.addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));

  vecmultigrid.addVectorOperator(&rhsOperator0, 0);

  Operator matrixOperator10(Operator::MATRIX_OPERATOR,
			    vecmultigrid.getFESpace(1),
			    vecmultigrid.getFESpace(0));

  Operator matrixOperator11(Operator::MATRIX_OPERATOR,
			    vecmultigrid.getFESpace(1),
			    vecmultigrid.getFESpace(1));

  matrixOperator10.addZeroOrderTerm(new Simple_ZOT);

  vecmultigrid.addMatrixOperator(&matrixOperator10, 1, 0);

  matrixOperator11.addZeroOrderTerm(new Simple_ZOT(-1.0));

  vecmultigrid.addMatrixOperator(&matrixOperator11, 1, 1);

  // ===== start adaption loop =====
  adapt->adapt();

  vecmultigrid.writeFiles(adaptInfo, true);

  return 0;
}


